/***********************************************************************

  File: donor_host.c

  Description:
       Interface program of advection model

       Usage:
       donor (invoked from AVS read (remote) module, used together
              with donor_host.c(CMMD), vanleer1.f(CMfortran))

  History:
  
  04/10/93      Gang Cheng      Created 


  Copyright Northeast Parallel Architectures Center 1993

*************************************************************************/

#include <stdio.h>
#include <math.h>
#include <avs/flow.h>
#include <avs/avs_data.h>
#include <avs/field.h>
#include <sys/types.h> 

#define debug 1

interface()
{
	int out_port;		/* temporary to hold the port number */
	int parm;		/* temporary to hold the parm number */

	/* Set the module name and type */
	AVSset_module_name("Advection_interface", MODULE_DATA);

	/* There are no input ports for this module */

	/* Create output ports for the result */
	out_port = AVScreate_output_port("Grid points", "integer");
        out_port = AVScreate_output_port("Time steps", "integer");
        out_port = AVScreate_output_port("time interval", "integer");

	parm = AVSadd_parameter("Grid points","integer", 40, 40, 1000);
	AVSconnect_widget(parm,"idial");
	parm = AVSadd_parameter("Time steps","integer", 100, 50, 1000);
	AVSconnect_widget(parm,"idial");
	parm = AVSadd_parameter("time interval","integer", 1, 1, 1000);
	AVSconnect_widget(parm,"idial");

	/* Add parameter: an enable/disable toggle for the scope */
        parm = AVSadd_parameter("start", "boolean", 0, 0, 1);
	parm = AVSadd_parameter("sleep", "boolean", 1, 0, 1);
	parm = AVSadd_parameter("step", "oneshot", 1, 0, 1);
	parm = AVSadd_parameter("abort", "oneshot", 0, 0, 1);

	/* There is no compute function for this module */
}

main(argc,argv)
int	argc;
char	*argv[];
{
  int points,times,tint,tcounter;
  int start,sleep,step,abort;
  int i,j;
  float total;

  float deltt=0.5, totmass,eps,deltx,t;
  int istep,istop,iostep,rtimes;
  float buffer; int num_nodes, pid = getpid();

  /* coroutine initilization */
  AVScorout_init(argc,argv,interface);

  AVScorout_set_sync(0); 

   while (1) {
     sleep = step  = 1; start = abort = 0;
     tint = 1;
 
     AVSparameter_visible("Grid points",1);
     AVSparameter_visible("Time steps",1);
     AVSparameter_visible("time interval",1);
     AVSparameter_visible("sleep",0);
     AVSparameter_visible("step",0);
     AVSparameter_visible("start",1);
     AVSparameter_visible("abort",0);
     
     do {
       AVScorout_wait();
       AVScorout_input(&points,&times,&tint,
		       &start,&sleep,&step,&abort);

       fprintf(stderr,"Grid points = %d Time steps = %d\n",points,times);
       if (tint > times) fprintf(stderr,
	 "Wrong input: time interval(%4d) > Time steps(%4d)\n",tint,times);
     } while  (!start || (tint>times));

     AVSparameter_visible("Grid points",0);
     AVSparameter_visible("Time steps",0);
     AVSparameter_visible("start",0);
     AVSparameter_visible("sleep",1);
     AVSparameter_visible("step",1);
     AVSparameter_visible("abort",1);

     /*    points = 32; */

     deltt = 50.0/(float)times;

     istop = times;
     istep = 0;
     do {
       
       if (sleep) AVScorout_wait();
       AVScorout_input(&points,&times,&tint,&start,&sleep,&step,&abort);      
       if (abort) break;
      
       if (tint >(istop-istep)) {
	 fprintf(stderr,"Wrong input: time interval(%4d) > Time steps left(%4d)\n",
		 tint,(istop-istep));
	 tint = istop-istep;
       }
       istep += tint;

       AVScorout_exec(); /* syncronization */
       AVScorout_output(points,times,tint);
       
       fprintf(stderr,"At time step: %3d\n",istep);
       
     } while (istep<istop);
   
     fprintf(stderr,"End of one cycle\n");
     /* inform downstream module end of a simulation cycle */
     tint = 0;
     AVScorout_exec(); /* syncronization */
     AVScorout_output(points,times,tint); 

     /* initilize parameter  */
     
     AVSmodify_parameter("Grid points",AVS_VALUE, 40, 40, 1000);
     AVSmodify_parameter("Time steps",AVS_VALUE, 100, 50, 1000);
     AVSmodify_parameter("time interval",AVS_VALUE, 1, 1, 1000);
     AVSmodify_parameter("start",AVS_VALUE, 0, 0, 1);
     AVSmodify_parameter("sleep",AVS_VALUE, 1, 0, 1);
     AVSmodify_parameter("step", AVS_VALUE, 1, 0, 1);
     AVSmodify_parameter("abort", AVS_VALUE, 0, 0, 1);
   }
}
